@@ -478,6 +478,8 @@ device_changed (Device          *device,
 		g_strfreev (uuids);
 		if (services != NULL)
 			g_hash_table_unref (services);
+	} else {
+		g_debug ("Unhandled property: %s", property);
 	}
 }
 
@@ -1362,7 +1364,7 @@ GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
  * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
  * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
  *
- * Returns a #GtkTreeModel of devices filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ * Returns a #GtkTreeModelFilter of devices filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
  *
  * Return value: (transfer full): a #GtkTreeModel object.
  **/
@@ -1407,7 +1409,7 @@ static gboolean adapter_filter(GtkTreeModel *model,
  * bluetooth_client_get_adapter_model:
  * @client: a #BluetoothClient object
  *
- * Returns a filtered #GtkTreeModel with only adapters present.
+ * Returns a #GtkTreeModelFilter with only adapters present.
  *
  * Return value: (transfer full): a #GtkTreeModel object.
  **/
@@ -1421,7 +1423,10 @@ GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
  * bluetooth_client_get_device_model:
  * @client: a #BluetoothClient object
  *
- * Returns a filtered #GtkTreeModel with only devices belonging to the default adapter listed. Note that the model will follow a specific adapter, and will not follow the default-adapter.
+ * Returns a #GtkTreeModelFilter with only devices belonging to the default adapter listed.
+ * Note that the model will follow a specific adapter, and will not follow the default adapter.
+ * Also note that due to the way #GtkTreeModelFilter works, you will probably want to
+ * monitor signals on the "child-model" #GtkTreeModel to monitor for changes.
  *
  * Return value: (transfer full): a #GtkTreeModel object.
  **/
@@ -1612,9 +1617,19 @@ connect_callback (GDBusProxy   *proxy,
 		  GAsyncResult *res,
 		  ConnectData  *conndata)
 {
+	GVariant *variant;
 	gboolean retval;
+	GError *error = NULL;
 
-	retval = device_call_connect_finish (DEVICE (proxy), res, NULL);
+	variant = g_dbus_proxy_call_finish (proxy, res, &error);
+	if (variant == NULL) {
+		retval = FALSE;
+		g_debug ("Connect failed: %s", error->message);
+		g_error_free (error);
+	} else {
+		g_variant_unref (variant);
+		retval = TRUE;
+	}
 
 	g_simple_async_result_set_op_res_gboolean (conndata->simple, retval);
 	g_simple_async_result_complete_in_idle (conndata->simple);
@@ -1630,12 +1645,28 @@ disconnect_callback (GDBusProxy   *proxy,
 		     ConnectData  *conndata)
 {
 	gboolean retval;
+	GError *error = NULL;
 
-	retval = device_call_disconnect_finish (DEVICE (proxy), res, NULL);
-
-	if (conndata->services != NULL) {
+	if (conndata->services == NULL) {
+		retval = device_call_disconnect_finish (DEVICE (proxy), res, &error);
+		if (retval == FALSE) {
+			g_debug ("Disconnect failed: %s", error->message);
+			g_error_free (error);
+		}
+	} else {
 		GDBusProxy *service;
 		BluetoothClient *client;
+		GVariant *variant;
+
+		variant = g_dbus_proxy_call_finish (proxy, res, &error);
+		if (variant == NULL) {
+			retval = FALSE;
+			g_debug ("Disconnect failed: %s", error->message);
+			g_error_free (error);
+		} else {
+			g_variant_unref (variant);
+			retval = TRUE;
+		}
 
 		client = (BluetoothClient *) g_async_result_get_source_object (G_ASYNC_RESULT (conndata->simple));
 		service = get_proxy_for_iface (DEVICE (proxy), conndata->services->data, client);
@@ -1729,6 +1760,7 @@ bluetooth_client_connect_service (BluetoothClient     *client,
 	GHashTable *table;
 	GtkTreeIter iter;
 	guint i;
+	gboolean res;
 
 	g_return_if_fail (BLUETOOTH_IS_CLIENT (client));
 	g_return_if_fail (device != NULL);
@@ -1737,12 +1769,10 @@ bluetooth_client_connect_service (BluetoothClient     *client,
 					    callback,
 					    user_data,
 					    bluetooth_client_connect_service);
+	res = FALSE;
 
-	if (get_iter_from_path (priv->store, &iter, device) == FALSE) {
-		g_simple_async_result_set_op_res_gboolean (simple, FALSE);
-		g_object_unref (simple);
-		return;
-	}
+	if (get_iter_from_path (priv->store, &iter, device) == FALSE)
+		goto bail;
 
 	if (cancellable != NULL) {
 		g_object_set_data_full (G_OBJECT (simple), "cancellable",
@@ -1760,18 +1790,21 @@ bluetooth_client_connect_service (BluetoothClient     *client,
 	if (proxy == NULL) {
 		if (table != NULL)
 			g_hash_table_unref (table);
-		g_simple_async_result_set_op_res_gboolean (simple, FALSE);
-		g_object_unref (simple);
-		return;
+		g_debug ("Device '%s' has a services table, but no proxy", device);
+		goto bail;
 	}
 
 	if (connect && table == NULL) {
-		if (proxy != NULL)
-			g_object_unref (proxy);
-		g_simple_async_result_set_op_res_gboolean (simple, FALSE);
-		g_object_unref (simple);
-		return;
-	} else if (connect) {
+		g_object_unref (proxy);
+		res = TRUE;
+		goto bail;
+	}
+
+	conndata = g_new0 (ConnectData, 1);
+	conndata->simple = simple;
+
+	if (connect) {
+		GDBusProxy *service;
 		const char *iface_name;
 
 		iface_name = NULL;
@@ -1781,32 +1814,26 @@ bluetooth_client_connect_service (BluetoothClient     *client,
 				break;
 			}
 		}
-		g_hash_table_unref (table);
 
 		if (iface_name == NULL) {
 			g_printerr("No supported services on the '%s' device\n", device);
-			if (proxy != NULL)
-				g_object_unref (proxy);
-			g_simple_async_result_set_op_res_gboolean (simple, FALSE);
-			g_object_unref (simple);
-			return;
+			g_hash_table_unref (table);
+			g_free (conndata);
+			g_object_unref (proxy);
+			goto bail;
 		}
-	}
 
-	conndata = g_new0 (ConnectData, 1);
-	conndata->simple = simple;
+		service = get_proxy_for_iface (DEVICE (proxy), iface_name, client);
 
-	if (connect) {
-		device_call_connect (DEVICE (proxy),
-				     NULL,
-				     (GAsyncReadyCallback) connect_callback,
-				     conndata);
-	} else if (table == NULL) {
-		device_call_disconnect (DEVICE (proxy),
-					NULL,
-					(GAsyncReadyCallback) disconnect_callback,
-					conndata);
-	} else {
+		g_dbus_proxy_call (G_DBUS_PROXY (service),
+				   "Connect",
+				   NULL,
+				   G_DBUS_CALL_FLAGS_NONE,
+				   -1,
+				   NULL,
+				   (GAsyncReadyCallback) connect_callback,
+				   conndata);
+	} else if (table != NULL) {
 		GDBusProxy *service;
 
 		conndata->services = g_hash_table_get_keys (table);
@@ -1819,13 +1846,25 @@ bluetooth_client_connect_service (BluetoothClient     *client,
 
 		g_dbus_proxy_call (G_DBUS_PROXY (service),
 				   "Disconnect",
-				   g_variant_new ("()"),
+				   NULL,
 				   G_DBUS_CALL_FLAGS_NONE,
 				   -1,
 				   NULL,
 				   (GAsyncReadyCallback) disconnect_callback,
 				   conndata);
+	} else if (table == NULL) {
+		device_call_disconnect (DEVICE (proxy),
+					NULL,
+					(GAsyncReadyCallback) disconnect_callback,
+					conndata);
 	}
+
+	return;
+
+bail:
+	g_simple_async_result_set_op_res_gboolean (simple, res);
+	g_simple_async_result_complete (simple);
+	g_object_unref (simple);
 }
 
 /**
